#!/usr/bin/perl

# This program fights for the users...

use strict;
use warnings;

use Getopt::Long;
use Device::SerialPort;
use English qw( $EUID -no_match_vars );
use vars qw($VERSION);

my $VERSION = '0.3.4';

my $wireless_interface = '';
my $iwlist_output      = '';

my $log_file_name = 'log_' . time() . '.log';
my $error_counter = 0;

my $parameter_show_help;
my $parameter_allow_log;

my $arduino_device;
my $serial_port;

# Check if the process is executed by a root user

die '[!] You must be root!', "\n" unless $EUID == 0;

# Get the ARGV parameters via Getopt::Long

GetOptions(
    'help' => \$parameter_show_help,
    'log'  => \$parameter_allow_log,
);

if ( defined($parameter_show_help) ) {

    print q {
ZLOIND }, $VERSION, q {
		
Usage: perl }, $0, q { [--options]

 --log:  allow the log tracking capability;
 --help: show this help.
 
SEE THE PERLDOC (perldoc }, $0, q {) FOR MORE INFORMATION

};
    exit;
}

# Show a nice ASCII art created by SSt

print q {
         ,-.
        / \  `.  __..-,O
       :   \ --''_..-'.'
       |    . .-' `. '.
       :     .     .`.'
        \     `.  /  ..
         \      `.   ' .
          `,       `.   \
         ,|,`.        `-.\
        '.||  ``-...__..-`
         |  |
         |__|
         /||\
        //||\\
       // || \\
    __//__||__\\__
   '--------------' SSt

ZLOIND v.}, $VERSION, "\n\n";

print '[*] Enter your serial port: ';

$serial_port = <STDIN>;

print '[*] Enter you wireless interface: ';

$wireless_interface = <STDIN>;

chomp( $serial_port, $wireless_interface );

# Trigger the communication with Arduino via serial port

$arduino_device = Device::SerialPort->new($serial_port)
  or die "[!] $!: $serial_port\n";

$arduino_device->baudrate(9600);
$arduino_device->databits(8);
$arduino_device->parity("none");
$arduino_device->stopbits(1);

print "\n", '[?] Start scanning zone...', "\n\n";

# Start the loop for the wireless network scanning

while (1) {

    my ( @output_lines, @ESSID, @ENCRYPTION );

    my $encry_counter    = 0;
    my $essid_counter    = 0;
    my $enc_data_counter = 0;

    # if enabled the log capability, make a log
    # for more information read the SEE ALSO in documentation for this code block

    ( defined($parameter_allow_log) )
      ? $iwlist_output = `iwlist $wireless_interface scan >> $log_file_name 2>&1`
      : $iwlist_output = `iwlist $wireless_interface scan 2>&1`;

    # Code block to check error messages from iwlist, if more than 10
    # error messages, the script stop working...

    if ( $iwlist_output =~ m{doesn}i ) {
        print "[!] Error output number $error_counter, your interface $wireless_interface, doesn't work!\n";

        $error_counter++;
    }
	
	if ( $error_counter >= 10 ) {
		$arduino_device->write('e');
		
		die "[!] Error your interface $wireless_interface, doesn't support scanning anymore!\n";
	}

    # Make 2 array, the first contains all ESSID of networks, the second one
    # will contains all the types of ENCRYTPTION, of each network.

    @output_lines = split( "\n", $iwlist_output );

    foreach my $i (@output_lines) {

        if ( $i =~ m{ESSID:"(.*?)"}ix ) {
            $ESSID[$essid_counter] = $1;

            $essid_counter++;
        }

        if ( $i =~ m{Encryption key:(on|off)}i ) {
            $ENCRYPTION[$encry_counter] = $1;

            $encry_counter++;
        }
    }

    # Scanning the array which contains all the encryption data

    foreach my $k (@ENCRYPTION) {

        if ( $k eq 'off' ) {
            print 'FREE WIRELESS NETWORK - ';

            $arduino_device->write('o');

            sleep(1);
        }
        elsif ( $k eq 'on' ) {
            print 'ENCRYPTED - ';

            $arduino_device->write('w');

            sleep(1);
        }

        print $ESSID[$enc_data_counter], "\n";

        $enc_data_counter++;
    }
}

__END__

=head1 NAME

ZLOIND

=head1 SYNOPSIS

perl zloind.pl [--options]

 --log:  allow the log tracking capability;
 --help: show this help.

=head1 DESCRIPTION

Zloind is a script to check from the tool "iwlist", the
access points detected by the system wireless interface. It captures
the ESSID and the kind of encryption. It'll send via serial port
an ASCII char that will be decoded by Arduino and executed by the board.
During the scanning of the access points, the script prints also a
notification in the shell screen. You can also enable the log
feature which allow you to make a list of all AP that iwlist detects.
If you want to use this script you need to install the module
Device::SerialPort, which is fundamental for the correct serial
communication with Arduino.

=head1 DEPENDENCIES

Getopt::Long ~ http://search.cpan.org/~enrys/POD2-IT-Getopt-Long/lib/POD2/IT/Getopt/Long.pm

Device::SerialPort ~ http://search.cpan.org/dist/Device-SerialPort/SerialPort.pm

English ~ http://search.cpan.org/~jesse/perl-5.14.1/lib/English.pm

vars ~ http://perldoc.perl.org/vars.html

=head1 SEE ALSO

Perlfaq8 ~ http://perldoc.perl.org/perlfaq8.html#How-can-I-capture-STDERR-from-an-external-command%3f

See the FAQ about Term modules at http://perldoc.perl.org/perlfaq8.html

See L<perlvar> for more information about L<English>

=head1 LICENSCE AND COPYRIGHT

Copyright (C) 2010 by sysxash <syxanash@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02111-1301,
USA.

On Debian systems, the complete text of the GNU General Public License
can be found in /usr/share/common-licenses/GPL-3.

=head1 AUTHOR

sysxash with his mind - <syxanash ( at ) gmail ( dot ) com>

=cut
